@@ -15,13 +15,12 @@ use failures qw{
 	index index::exists
 };
 
-use Tie::IxHash;
-use Tie::IxHash::Extension;
+use Hash::Ordered;
 use PDL::Basic qw(sequence);
 use PDL::Core qw(pdl null);
 use Data::Perl     ();
 use Data::Perl::Collection::Array;
-use List::AllUtils qw(each_arrayref pairgrep pairkeys pairmap);
+use List::AllUtils qw(each_arrayref pairgrep pairkeys pairmap pairwise);
 use List::MoreUtils 0.423;
 use PDL::Primitive ();
 use PDL::Factor    ();
@@ -132,7 +131,7 @@ around BUILDARGS($orig, $class : @args) {
 
 sub _trait_namespace { 'Data::Frame::Role' } # override for MooX::Traits
 
-has _columns => ( is => 'ro', default => sub { Tie::IxHash->new; } );
+has _columns => ( is => 'ro', default => sub { Hash::Ordered->new; } );
 
 has _row_names => ( is => 'rw', predicate => 1 );
 
@@ -205,9 +204,8 @@ method string( $row_limit = 10 ) {
 }
 
 
-sub number_of_columns {
-	my ($self) = @_;
-	$self->_columns->Length;
+method number_of_columns() {
+	return scalar($self->_columns->keys);
 }
 
 *ncol   = \&number_of_columns;
@@ -291,11 +289,11 @@ method at (@rest) {
 sub select_columns { shift->_select_columns(@_); }
 
 method exists ($col_name) {
-    $self->_columns->EXISTS($col_name);
+    $self->_columns->exists($col_name);
 }
 
 method delete ($col_name) {
-    $self->_columns->Delete($col_name);
+    $self->_columns->delete($col_name);
 }
 
 method rename ((HashRef | CodeRef) $href_or_coderef) {
@@ -334,7 +332,7 @@ method set ($indexer, $data) {
 
     if ( $self->exists($name) ) {
         $self->_column_validate( $name => $data );
-        $self->_columns->Push( $name => $data );
+        $self->_columns->set( $name => $data );
     }
     else {
         $self->add_column( $name, $data );
@@ -346,8 +344,7 @@ method set ($indexer, $data) {
 method isempty () { $self->nrow == 0; }
 
 # supports negative indices
-sub nth_column {
-	my ($self, $index) = @_;
+method nth_column($index) {
 	failure::index->throw({
 			msg => "requires index",
 			trace => failure->croak_trace
@@ -357,7 +354,7 @@ sub nth_column {
 			trace => failure->croak_trace,
 		}) if $index >= $self->number_of_columns;
 	# fine if $index < 0 because negative indices are supported
-	$self->_columns->Values( $index );
+	return ($self->_columns->values)[$index];
 }
 
 
@@ -373,16 +370,18 @@ method column_names(@rest) {
       : @rest;
 
 	if( @colnames ) {
-		try {
-			$self->_columns->RenameKeys( @colnames );
-		} catch {
+        unless (@colnames == $self->length) {
 			failure::columns::length->throw({
 					msg => "incorrect number of column names",
 					trace => failure->croak_trace,
-				}) if $@->$_isa('failure::keys::number');
-		};
+				});
+        }
+        # rename column names
+        my @values = $self->_columns->values;
+        $self->_columns->clear;
+        $self->_columns->push( List::AllUtils::zip( @colnames, @values ) );
 	}
-	[ $self->_columns->Keys ];
+	return [ $self->_columns->keys ];
 }
 
 *col_names = \&column_names;
@@ -437,13 +436,12 @@ sub _make_actual_row_names {
 }
 
 
-sub column {
-	my ($self, $colname) = @_;
+method column($colname) {
 	failure::column::exists->throw({
 			msg => "column $colname does not exist",
 			trace => failure->croak_trace,
 		}) unless $self->exists( $colname );
-	$self->_columns->FETCH( $colname );
+	return $self->_columns->get( $colname );
 }
 
 sub _column_validate {
@@ -466,8 +464,7 @@ sub _column_validate {
 }
 
 
-sub add_columns {
-	my ($self, @columns) = @_;
+method add_columns(@columns) {
 	failure::columns::unbalanced->throw({
 			msg => "uneven number of elements for column specification",
 			trace => failure->croak_trace,
@@ -490,7 +487,7 @@ sub add_column {
 	$data = PDL::SV->new( $data ) if ref $data eq 'ARRAY';
 
 	$self->_column_validate( $name => $data);
-	$self->_columns->Push( $name => $data );
+	$self->_columns->push( $name => $data );
 }
 
 # R
@@ -499,7 +496,7 @@ sub add_column {
 # $ sequence(10,4)->dice(X,[0,1,1,0])
 
 method select_rows(@rest) {
-    my $indexer = iloc(@rest);
+    my $indexer = indexer_i(@rest);
     return $self unless defined $indexer;
 
     my $indices = $self->_rindexer_to_indices($indexer);
@@ -637,7 +634,7 @@ sub _column : lvalue     { my $col = shift->column(@_);     return $col; }
 sub _nth_column : lvalue { my $col = shift->nth_column(@_); return $col; }
 
 method _select_columns (@rest) : lvalue {
-    my $indexer = loc(@rest);
+    my $indexer = indexer_s(@rest);
     return $self if ( not defined $indexer or $indexer->indexer->length == 0 );
 
     my $indices      = $self->_cindexer_to_indices($indexer);
@@ -672,9 +669,8 @@ classmethod _check_slice_args (@rest) {
     return ( $row_indexer, $column_indexer );
 }
 
-method slice (@rest) : lvalue {
-    my ( $rindexer, $cindexer ) = $self->_check_slice_args(@_);
-
+method slice(@rest) : lvalue {
+    my ( $rindexer, $cindexer ) = $self->_check_slice_args(@rest);
     my $new_df = $self->select_rows($rindexer);
     $new_df = $new_df->select_columns($cindexer);
     return $new_df;
@@ -822,10 +818,10 @@ method which (:$bad_to_val=undef, :$ignore_both_bad=true) {
     my $coordinates = [ 0 .. $self->ncol - 1 ]->map(
         fun($cidx)
         {
-            my $column = $self->at( iloc( [$cidx] ) );
+            my $column = $self->at( indexer_i( [$cidx] ) );
             my $both_bad =
                 $self->DOES('Data::Frame::Role::CompareResult')
-              ? $self->both_bad->at( iloc( [$cidx] ) )
+              ? $self->both_bad->at( indexer_i( [$cidx] ) )
               : undef;
 
             if ( defined $bad_to_val ) {
@@ -984,7 +980,7 @@ Data::Frame - data frame implementation
 
 =head1 VERSION
 
-version 0.004_001
+version 0.004_002
 
 =head1 STATUS
 
@@ -1145,8 +1141,8 @@ If two arguments are given, it would treat the arguments for row
 indexer and column indexer respectively to get the cell value.
 
 If a given argument is non-indexer, it would try guessing whether the
-argument is numeric or not, and coerce it by either C<loc()> or
-C<iloc()>.
+argument is numeric or not, and coerce it by either C<indexer_s()> or
+C<indexer_i()>.
 
 =head2 exists
 
@@ -1173,7 +1169,7 @@ In-place rename columns.
 
 Returns a new data frame object which has the columns selected by C<$indexer>.
 
-If a given argument is non-indexer, it would coerce it by C<loc()>.
+If a given argument is non-indexer, it would coerce it by C<indexer_s()>.
 
 =head2 set
 
@@ -1399,7 +1395,7 @@ changing a subset of the raw data frame. For example,
     $df->slice($row_indexer, $column_indexer) .= $piddle;
 
 If a given argument is non-indexer, it would try guessing if the argument
-is numeric or not, and coerce it by either C<loc()> or C<iloc()>.
+is numeric or not, and coerce it by either C<indexer_s()> or C<indexer_i()>.
 
 =head2 assign
 
