@@ -8,7 +8,7 @@ use namespace::autoclean;
 use Eval::Quosure 0.001;
 use Types::Standard;
 
-use Data::Frame::Indexer qw(loc);
+use Data::Frame::Indexer qw(indexer_s);
 
 
 method eval_tidy ($x) {
@@ -19,7 +19,7 @@ method eval_tidy ($x) {
 
     my $expr = $is_quosure ? $x->expr : $x;
     if ( $self->exists($expr) ) {
-        return $self->at( loc($expr) );
+        return $self->at( indexer_s($expr) );
     }
 
     my $quosure = $is_quosure ? $x : Eval::Quosure->new( $expr, 1 );
@@ -51,7 +51,7 @@ Data::Frame::Partial::Eval - Partial class for data frame's eval method
 
 =head1 VERSION
 
-version 0.004_001
+version 0.004_002
 
 =head1 SYNOPSIS
 
