@@ -6,6 +6,10 @@ use Data::Frame::Role;
 use namespace::autoclean;
 
 package Tie::Data::Frame {
+    use Types::PDL qw(Piddle);
+    use Types::Standard qw(ArrayRef Value);
+    use Type::Params;
+
     sub new {
         my ($class, $object) = @_;
         return bless( { _object => $object }, $class);
@@ -18,8 +22,17 @@ package Tie::Data::Frame {
 
     sub object { $_[0]->{_object} }
 
+    sub _check_key {
+        my $self = shift;
+        state $check = Type::Params::compile(Value | ArrayRef | Piddle);
+        my ($key) = $check->(@_);
+        return $key;
+    }
+
     sub STORE {
         my ( $self, $key, $val ) = @_;
+        $key = $self->_check_key($key);
+
         if ( Ref::Util::is_ref($key) ) {
             $self->object->slice($key) .= $val;
         } else {
@@ -29,6 +42,8 @@ package Tie::Data::Frame {
 
     sub FETCH {
         my ( $self, $key ) = @_;
+        $key = $self->_check_key($key);
+
         if ( Ref::Util::is_ref($key) ) {
             return $self->object->slice($key);
         } else {
@@ -36,9 +51,15 @@ package Tie::Data::Frame {
         }
     }
 
+    sub EXISTS {
+        my ($self, $key) = @_;
+        return $self->object->exists($key);
+    }
+
     sub FIRSTKEY {
         my ($self) = @_;
-        return shift @{$self->{_list}};
+        $self->{_list} = [ @{$self->object->names} ];
+        return $self->NEXTKEY;
     }
 
     sub NEXTKEY {
@@ -50,8 +71,10 @@ package Tie::Data::Frame {
 use overload (
     '%{}' => sub {    # for working with Tie::Data::Frame
         my ($self)   = @_;
+
+        # This is brittle as we are depending on an private thing of Moo... 
         my ($caller) = caller();
-        if ( $caller =~ /^Method::Generate::Accessor::/ ) {
+        if ( $caller eq 'Method::Generate::Accessor::_Generated' ) {
             return $self;
         }
         return ( $self->_tie_hash // $self );
@@ -81,7 +104,7 @@ Data::Frame::Partial::Sugar - Partial class for data frame syntax sugar
 
 =head1 VERSION
 
-version 0.004_001
+version 0.004_002
 
 =head1 SYNOPSIS
 
